@@ -1,5 +1,16 @@
+# last modified 5 Dec 04 by J. Fox
+
 "polyserial" <-
-function(x, y, ML=FALSE, control=list(), std.err=FALSE){
+function(x, y, ML=FALSE, control=list(), std.err=FALSE, maxcor=.9999){
+  f <- function(pars){
+    rho <- pars[1]
+    if (abs(rho) > maxcor) rho <- sign(rho)*maxcor
+    cts <- if (length(pars) == 1) c(-Inf, cuts, Inf)
+           else c(-Inf, pars[-1], Inf)
+    tau <- (matrix(cts, n, s+1, byrow=TRUE) - matrix(rho*z, n, s+1))/
+             sqrt(1 - rho^2)
+    - sum(log(dnorm(z)*(pnorm(tau[cbind(indices, y+1)]) - pnorm(tau[cbind(indices, y)]))))
+    }
   if (!is.numeric(x)) stop("x must be numeric")
   valid <- complete.cases(x, y)
   x <- x[valid]
@@ -12,13 +23,18 @@ function(x, y, ML=FALSE, control=list(), std.err=FALSE){
   cuts <- qnorm(cumsum(tab)/n)[-s]
   y <- as.numeric(as.factor(y))
   rho <- sqrt((n - 1)/n)*sd(y)*cor(x, y)/sum(dnorm(cuts))
-  if (!ML) return(rho)
-  f <- function(pars){
-    rho <- pars[1]
-    cuts <- c(-Inf, pars[-1], Inf)
-    tau <- (matrix(cuts, n, s+1, byrow=TRUE) - matrix(rho*z, n, s+1))/
-             sqrt(1 - rho^2)
-    - sum(log(dnorm(z)*(pnorm(tau[cbind(indices, y+1)]) - pnorm(tau[cbind(indices, y)]))))
+  if (!ML) {
+    if (std.err){
+      result <- optim(rho, f, control=control, hessian=TRUE, method="BFGS")
+      result <- list(type="polyserial",
+                   rho=result$par,
+                   var=1/result$hessian,
+                   n=n,
+                   ML=FALSE)
+      class(result) <- "polycor"
+      return(result)
+      }
+    else return(rho)
     }
   result <- optim(c(rho, cuts), f, control=control, hessian=std.err)
   if (std.err){
@@ -26,10 +42,10 @@ function(x, y, ML=FALSE, control=list(), std.err=FALSE){
                    rho=result$par[1],
                    cuts=result$par[-1],
                    var=solve(result$hessian),
-                   n=n)
+                   n=n,
+                   ML=TRUE)
     class(result) <- "polycor"
     return(result)
     }
   else return(as.vector(result$par[1]))
   }
-
